/*
    Calculator.h

    Copyright  1996 Levin-Delson Software.  All Rights Reserved.

    $Header: /Bedazzle/dvlp/Calculator/RCS/Calculator.h,v 1.12 1996/09/18 07:30:07 adamld Exp $
*/

#ifndef CALCULATOR_H
#define CALCULATOR_H

#include <AppKit.h>
#include <InterfaceKit.h>
#include <SupportKit.h>

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <float.h>

#include "CalcWindow.h"
#include "CalcKeyCaps.h"
#include "CalcRevision.h"

#define CALC_SIGNATURE 'CL8R'

#define CALC_NAMEOF_WINDOW        "Calculator"
#define CALC_NAMEOF_BACKGROUND    "CalcBackground"
#define CALC_NAMEOF_DISPLAY       "CalcDisplay"

// This doesn't limit how many Calculators can be run, only the magnitude of the numeric ID.
#define CALC_NUMBEROF_INSTANTIATIONS 999

// These constants define how far into the screen the window will be placed.
#define CALC_MARGINPERCENT_H .02
#define CALC_MARGINPERCENT_V .05

// GUI layout constants.
#define CALC_WIDTHOF_BUTTON      24
#define CALC_WIDTHOF_GAP        ( CALC_WIDTHOF_BUTTON / 3 )
#define CALC_HEIGHTOF_BUTTON     24
#define CALC_HEIGHTOF_GAP       ( CALC_HEIGHTOF_BUTTON / 3 )

#define CALC_BUTTONS_PER_ROW      6
#define CALC_BUTTONS_PER_COLUMN   4

#define CALC_NUMBEROF_BUTTONS ( ( CALC_BUTTONS_PER_ROW ) * ( CALC_BUTTONS_PER_COLUMN ) )

// The maximum number of digits, internal and display use.
#define CALC_INTERNAL_DIGITS 31
#define CALC_DISPLAY_DIGITS  11

// Calculator state machine constants.
#define CALC_MODE_OPERAND1     ( 1UL <<  0 )
#define CALC_MODE_OPERAND2     ( 1UL <<  1 )
#define CALC_MODE_ERROR        ( 1UL <<  2 )
#define CALC_MODE_FIRST        ( 1UL <<  3 )
#define CALC_MODE_PASTE        ( 1UL << 31 )

// Local prototypes.
bool queryAppCalcData( void *voidAppInfo, void *voidCalcData );
int dtostr( double result, char *fullString );
void enableButton( BMessenger *bMessenger, char *buttonName, bool state );

/*
    The CalcData structure is used to pass data to the QueryAppCalcData routine, which can
    only take one argument as it is called from BList's DoForEach.
*/
struct CalcData
{
    ulong cd_ThisCalculator;
    ulong cd_OriginalNumLocked;
};

class Calculator : public BApplication
{
    public:
                      Calculator();
                     ~Calculator();
        virtual void  AboutRequested( void );
        virtual void  MessageReceived( BMessage *message );
    private:
        ulong       thisCalculator;                     // The "number" of this instantiation.
        ulong       mode;                               // State machine mode; one of CALC_MODE_...
        char        fullString[ CALC_INTERNAL_DIGITS + 1];   // The input string, including sign.
        char       *string;                             // The input string, less the sign.
        int         digits;                             // The number of digits in the input string.
        double      operand1;                           // The first operand (second is only needed locally.
        ulong       operation;                          // The math operation to be performed.
        double      memory;                             // The Calculator has one memory.
        CalcWindow *calcWindow;                         // The window.
        BMessenger *windowMessenger;                    // A connection to the window's handler.
        ulong       originalNumLocked;                  // The original state of the Num Lock.
};

#endif
